#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"
#include "select_screen.h"
#include "cust_details.h"

DataEntry *merge_cust = NULL;

static GtkCList *merge_cust_list;

static int look_mode = 0;

/*----------------------------------------------------------------------------------------------------
::merge cust screen
----------------------------------------------------------------------------------------------------*/
enum  {
	MERGE_CUST_LIST,

	MERGE_CUST_OK_BTN,
	MERGE_CUST_QUIT_BTN
};

static void merge_cust_populate (PGresult *cust_res)
{
	int i, nTuples;
	char *cust_list_row[6];

	gtk_clist_freeze (merge_cust_list);
	gtk_clist_clear (merge_cust_list);

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		cust_list_row[0] = PQgetvalue (res, i, 0);
		cust_list_row[1] = PQgetvalue (res, i, 1);
		cust_list_row[2] = PQgetvalue (res, i, 2);
		cust_list_row[3] = PQgetvalue (res, i, 3);
		cust_list_row[4] = PQgetvalue (res, i, 4);
		cust_list_row[5] = nice_date (PQgetvalue (res, i, 5));
		gtk_clist_append (GTK_CLIST(merge_cust_list), cust_list_row);
	}
	gtk_clist_thaw (merge_cust_list);

	// do NOT clear the result set here - twas passed in elsewhere !
}

static void merge_cust_quit_callback (GtkWidget *widget, gpointer data)
{
	gtk_widget_hide (GTK_WIDGET(merge_cust));
	select_screen_redisplay ();
}

static void merge_cust_callback (GtkWidget *widget, gpointer data)
{
	int i, j;
	char *text, *major;
	GtkCListRow *clist_row;


	if (!look_mode)
	{
	PQclear (execSQL ("begin"));
	// delete the old ones

	PQclear (execSQL ("delete from merge_cust"));

	for (i = 0; i < merge_cust_list->rows; i++)
	{
		clist_row = (g_list_nth (merge_cust_list->row_list, i))->data;

		if (clist_row->state == GTK_STATE_SELECTED)
		{
			printf ("Row %d is selected\n", i);
			gtk_clist_get_text (GTK_CLIST(merge_cust_list), i,0,&text);
			printf ("  id = %s\n", text);

			PQclear (execSQL("insert into merge_cust select * from customers "
				"where custnum = %s", text));
		}
	}

	res = execSQL ("select * from merge_cust");
	merge_cust_populate (res);
	PQclear (res);
	PQclear (execSQL ("commit"));
	look_mode = 1;
	gtk_clist_set_selection_mode (GTK_LIST(merge_cust_list), GTK_SELECTION_BROWSE);
	message_box ("Now select which one to merge all of these into ...");
	}
	else // select 1 to merge into mode
	{
	PQclear (execSQL ("begin"));
		for (i = 0; i < merge_cust_list->rows; i++)
		{
			clist_row = (g_list_nth (merge_cust_list->row_list, i))->data;

			if (clist_row->state == GTK_STATE_SELECTED)
			{
				gtk_clist_get_text (GTK_CLIST(merge_cust_list), i,0,&major);
				printf (" Merge into id = %s\n", major);

				for (j = 0; j < merge_cust_list->rows; j++)
				{
					clist_row = (g_list_nth (merge_cust_list->row_list, j))->data;
					if (j == i) continue;

					gtk_clist_get_text (GTK_CLIST(merge_cust_list), j,0,&text);
					PQclear (execSQL ("update orders set custnum = %s where custnum = %s",
						major, text));
					PQclear (execSQL ("delete from customers where custnum = %s",
						text));
				}

				break;	// all done !
			}
		}

		PQclear (execSQL ("commit"));
		gtk_widget_hide (GTK_WIDGET(merge_cust));
		select_screen_redisplay ();
	}
}

void select_merge_list_select (GtkWidget *widget, GdkEvent *evt, int row, int col, gpointer data)
{
	if (!look_mode) return;

	printf ("Selected row %d\n", row);
}

void do_merge_cust (PGresult *cust_res)
{
	DataEntry *data_entry;
	GtkWidget *window;
	static char *titles[] =
	{
        "ID","Name","Address","Phone","Suburb","Last Date"
	};

	look_mode = 0;

	if (!merge_cust)
	{
		data_entry = DATA_ENTRY(data_entry_new ("Select Customers to Merge", 20, 20, 0,0));
		window = (GtkWidget *) data_entry;
		gtk_widget_realize (window);

		data_entry_add_label_button (data_entry, MERGE_CUST_OK_BTN, "Merge", 
				GTK_SIGNAL_FUNC(merge_cust_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MERGE_CUST_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(merge_cust_quit_callback), BUTTON_QUIT);

        merge_cust_list = gtk_clist_new_with_titles (6, titles);
        // TODO - set the correct colun widths
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 0, 40);
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 1, 120);
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 2, 150);
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 3, 90);
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 4, 110);
        gtk_clist_set_column_width (GTK_CLIST(merge_cust_list), 5, 160);
		gtk_clist_set_selection_mode (GTK_CLIST(merge_cust_list), GTK_SELECTION_MULTIPLE);

        gtk_signal_connect (GTK_OBJECT (merge_cust_list), "select_row",
			(GtkSignalFunc) select_merge_list_select, data_entry);

        gtk_widget_set_usize (GTK_WIDGET(merge_cust_list), 750, 400);

        data_entry_add_widget (data_entry, MERGE_CUST_LIST, GTK_WIDGET(merge_cust_list), 
			0,0,10,1);
		gtk_list_set_selection_mode (GTK_LIST(merge_cust_list), GTK_SELECTION_MULTIPLE);
  
		merge_cust = data_entry;
	}

	gtk_list_set_selection_mode (GTK_LIST(merge_cust_list), GTK_SELECTION_MULTIPLE);
	if (!GTK_WIDGET_VISIBLE (merge_cust))
		gtk_widget_show (GTK_WIDGET(merge_cust));

	merge_cust_populate (cust_res);
}

void merge_cust_redisplay (void)
{
	if (!GTK_WIDGET_VISIBLE (merge_cust))
		gtk_widget_show (GTK_WIDGET(merge_cust));
}
